/* 
 *  Copyright (c) 2010, 2011 Carlos Pais <freemind@lavabit.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "properties.hpp"
#include <QDir>


PropertiesDialog::PropertiesDialog(QList<QModelIndex>& indexes, ScopeModel& filesystem, QWidget * p_parent)
    : QDialog(p_parent)
{
    setupUi(this);
    
    m_SettingChanged = false;
    fs = NULL;
    m_filesystem = &filesystem;
    m_indexes = indexes;
    
    if ( m_indexes.size() > 1 )
    {
        t_name->hide();
        l_vol->hide();
        l_vol_cont->hide();
        l_openwith->hide();
        cb_openwith->hide();
        l_accessed->hide();
        l_accessed_cont->hide();
        l_modified->hide();
        l_modified_cont->hide();
        l_type->hide();
        l_type_cont->hide();
        resize(width(), height()/2);
    }
    
    setInfo();
    
    connect(t_name, SIGNAL(textEdited(const QString &)), SLOT(nameEdited(const QString &)));
    connect(&timer, SIGNAL(timeout()), SLOT(setFolderSize()));
    
    //Disables the permissions, so the user can't change them
    enablePermissions(false);
}


PropertiesDialog::~PropertiesDialog()
{
    if (timer.isActive())
        timer.stop();
    
    if ( fs != NULL ){
        if ( fs->isRunning() ){
            fs->stopProcessing();
            fs->wait();
        }
        
        fs->deleteLater();
    }

}

void PropertiesDialog::setInfo()
{
    if ( m_indexes.size() > 1 )
    {
        QString name = QString("%1 Files").arg(m_indexes.size());
        l_name->setText(name);
        t_path->setText( m_filesystem->fileInfo(m_indexes[0]).canonicalPath() );
        fs = new FolderSize(m_indexes, m_filesystem);
        fs->start();
        timer.start(100);
    }
    else if ( m_indexes.size() == 1 )
    {
        m_index = m_indexes[0];
        m_fileInfo = m_filesystem->fileInfo(m_index);
        qint64 size = 0;
        if ( m_fileInfo.isDir() ){
            fs = new FolderSize(m_fileInfo.absolutePath());
            fs->start();
            timer.start(100);
        }
        else
            size = m_filesystem->size(m_index);

        l_name->setPixmap(m_filesystem->icon(m_fileInfo).pixmap(32, 32));
        t_name->setText( m_filesystem->fileName(m_index) );
        if (! m_fileInfo.isWritable() )
            t_name->setReadOnly(true);
        
        l_size_cont->setText(QString::number(size/1024.0, 'f', 2) + QString(" KiB"));
        t_path->setText( m_filesystem->fileInfo(m_index).canonicalPath() );
        l_modified_cont->setText(m_filesystem->lastModified(m_index).toString());
        l_accessed_cont->setText(m_filesystem->fileInfo(m_index).lastRead().toString());
        setPermissions( m_fileInfo.permissions() );
    }
    
    


}

void PropertiesDialog::setName(QString name)
{
    t_name->setText(name);
}

void PropertiesDialog::setSize(qint64 size)
{
    double kib;
    
    //set in KiB
    l_size_cont->setText(QString::number((double)size/1024) + QString(" KiB"));
}

void PropertiesDialog::setLastModified(QDateTime date)
{
    l_modified_cont->setText(date.toString());
}

void PropertiesDialog::setLastRead(QDateTime date)
{
    l_accessed_cont->setText(date.toString());
}

void PropertiesDialog::setPath(QString path)
{
    t_path->setText(path);
}

void PropertiesDialog::setPerm(QString name)
{
    t_num->setText(name);
}

void PropertiesDialog::setPermissions(QFlags<QFile::Permission> perms)
{
    int value_tmp = 0;
    int value = 0;
    
    //User/Owner permissions
    if( perms.testFlag(QFile::ReadUser) )
    {
        o_read->setCheckState(Qt::Checked);
        value_tmp += 4;
    }
    if(perms.testFlag(QFile::WriteUser))
    {
        o_write->setCheckState(Qt::Checked);
        value_tmp += 2;
    }
    if(perms.testFlag(QFile::ExeUser))
    {
        o_exec->setCheckState(Qt::Checked);
        value_tmp += 1;
    }
       
    value_tmp *= 100;
    value += value_tmp;
    
    //Group permissions
    value_tmp = 0;
    
    if(perms.testFlag(QFile::ReadGroup))
    {
        g_read->setCheckState(Qt::Checked);
        value_tmp += 4;
    }
    if(perms.testFlag(QFile::WriteGroup))
    {
        g_write->setCheckState(Qt::Checked);
        value_tmp += 2;
    }
    if(perms.testFlag(QFile::ExeGroup))
    {
        g_exec->setCheckState(Qt::Checked);
        value_tmp += 1;
    }    
    value_tmp *= 10;
    value += value_tmp;
    
    //Public/Other permissions
    value_tmp = 0;
    
    if(perms.testFlag(QFile::ReadOther))
    {
        p_read->setCheckState(Qt::Checked);
        value_tmp += 4;
    }
    if(perms.testFlag(QFile::WriteOther))
    {
        p_write->setCheckState(Qt::Checked);
        value_tmp += 2;
    }
    if(perms.testFlag(QFile::ExeOther))
    {
        p_exec->setCheckState(Qt::Checked);
        value_tmp += 1;
    } 
      
    value += value_tmp;
    
    t_num->setText(QString("%1").arg(value));
    
}

void PropertiesDialog::makeConnections()
{
}

void PropertiesDialog::enablePermissions(bool enable)
{
    o_read->setEnabled(enable);
    o_write->setEnabled(enable);
    o_exec->setEnabled(enable);
    g_read->setEnabled(enable);
    g_write->setEnabled(enable);
    g_exec->setEnabled(enable);
    p_read->setEnabled(enable);
    p_write->setEnabled(enable);
    p_exec->setEnabled(enable);
    t_num->setEnabled(enable);
}


/* SLOTS */
void PropertiesDialog::setFolderSize()
{
    l_size_cont->setText(QString::number(fs->getTotalSize()/1048576.0, 'f', 2) + " MiB");
    

    
    if ( fs->isFinished() )
    {
        timer.stop();
        
        
        QString name("");
        if ( fs->getTotalFolders() > 0 )
            name += QString("%1 %2").arg(fs->getTotalFolders()).arg(tr("Folder(s)"));
        if ( fs->getTotalFiles() > 0 )
        {
            if ( ! name.isEmpty() )
                name += ", ";
            name += QString("%1 %2").arg(fs->getTotalFiles()).arg(tr("File(s)"));
        }
            
        
        l_name->setText(name);
        
        delete fs;
        fs = NULL;
    }
}

void PropertiesDialog::nameEdited(const QString & newName)
{
    m_SettingChanged = true;
}


void PropertiesDialog::applyChanges()
{
    if ( t_name->text() != m_fileInfo.fileName() )
        QDir::current().rename(m_fileInfo.fileName(), t_name->text());
}



