/* 
 *  Copyright (c) 2010, 2011 Carlos Pais <freemind@lavabit.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "pathline.hpp"


PathLine::PathLine(QFileSystemModel *m, QWidget *parent)
    : LineEdit(parent)
{
    model = m;
    
    completer_model = new FileSystemModel(this);
    
    completer = new Completer(completer_model, this);
    completer->setCompletionMode(QCompleter::PopupCompletion);
    setCompleter(completer);
    
    connect(completer->popup(), SIGNAL(activated(const QModelIndex &)), SLOT(popupClicked(const QModelIndex &)));
    connect(completer->popup(), SIGNAL(clicked(const QModelIndex &)), SLOT(popupClicked(const QModelIndex &)));

    connect(this, SIGNAL(returnPressed()), SLOT(returnPressed()));
    
    color_in_use = QApplication::palette().text().color();

}

void PathLine::popupClicked(const QModelIndex & index)
{
    returnPressed();
}


void PathLine::addButtons(QString p)
{
    
    path = p;
    int i=0, j=0;
    QString current_path;
    QSize size;
    
    
    foreach(QString name, path.split(QDir::separator()))
    {
        if ( ! name.isEmpty() || i == 0)
        {
            if (i == 0)
            {
                current_path.append("/");
                name = "/";
            }
            else
                current_path.append(name + "/");
            
            QPushButton *pbtn = new QPushButton(this);
            //pbtn->setStyleSheet("* { color: white; background-color: transparent ; border-color: 2px solid white; opacity: 100}");
            pbtn->setStyleSheet(QString("color: %1; background-color: transparent ; border-color: 2px solid white; opacity: 100;\
                                          margin:0px").arg(color_in_use.name()));
            pbtn->setText(name);
            pbtn->setFlat(true);
            pbtn->setCursor(Qt::ArrowCursor);
            
            size = pbtn->sizeHint();
            size.setWidth(name.length()*9);
            pbtn->setMaximumSize(size);
            pbtn->adjustSize();
            
            connect(pbtn,  SIGNAL(clicked()), SLOT(goTo()));
            addWidget(pbtn, LeftSide);
            buttons.append(pbtn);
            buttonsPath.insert(pbtn, current_path);
            
            if (hasDirs(current_path))
            {
                QPushButton *arrow = new QPushButton(this);
                connect(arrow,  SIGNAL(clicked()), SLOT(showMenu()));
                //connect(arrow,  SIGNAL(focusOutEvent( QFocusEvent * e )), SLOT(release(QFocusEvent * e)));
                arrow->setIcon(QIcon("/usr/share/icons/oxygen/16x16/actions/arrow-right.png"));
                arrow->setFlat(true);
                arrow->setCursor(Qt::ArrowCursor);
                
                size = arrow->sizeHint();
                size.setWidth(16);
                arrow->setMaximumSize(size);
                buttonsPath2.insert(arrow, current_path);
                buttons.append(arrow);
                addWidget(arrow, LeftSide);
            }   
           
            i++;
        }
    }
    
}

bool PathLine::hasDirs(QString path)
{
    QDir dir(path);
    QDir::Filters flags (QDir::AllDirs |
                     QDir::System |
                     QDir::NoDotAndDotDot);
    
    if (dir.entryList(flags).length() > 0)
        return true;
        
    return false;
}

void PathLine::release()
{
    //QPushButton * arrow = qobject_cast<QPushButton*>(sender());
    current_button_pressed->setIcon(QIcon("/usr/share/icons/oxygen/16x16/actions/arrow-right.png"));
    mainmenu->deleteLater();
    
}

void PathLine::showMenu()
{

    QPushButton * arrow = qobject_cast<QPushButton*>(sender());
    current_button_pressed = arrow;
    arrow->setIcon(QIcon("/usr/share/icons/oxygen/16x16/actions/arrow-down.png"));
    QMenu  menu  (this);
    
    mainmenu = new QMenu ();
    connect(mainmenu,  SIGNAL(aboutToHide()), SLOT(release()));
    
    QWidgetAction *a_files = new QWidgetAction(&menu);
    QTreeWidget *tree = new QTreeWidget();
    tree->header()->setVisible(false);
    tree->setStyleSheet(QString("color: %1; opacity: 100").arg(color_in_use.name()));
    connect(tree, SIGNAL(itemClicked ( QTreeWidgetItem *, int)),
            SLOT(goTo( QTreeWidgetItem *, int)));
    
    //QScrollArea to include the QMenu
    QScrollArea *scroll = new QScrollArea ();
    scroll->setWidget(tree);
    
    //get files
    QString path = buttonsPath2.value(arrow);
    QDir dir(path);
    
    QDir::Filters flags (QDir::AllDirs |
                         QDir::System |
                         QDir::NoDotAndDotDot);

    foreach(QString file, dir.entryList(flags))
    {
        //menu.addAction(file);
        QTreeWidgetItem *item = new QTreeWidgetItem(QStringList(file));
        item->setIcon(0, QIcon("/usr/share/icons/oxygen/16x16/places/folder.png"));
        tree->addTopLevelItem(item);
    }

    
    a_files->setDefaultWidget (scroll);
    mainmenu->addAction(a_files);
    
    arrow->setMenu(mainmenu);
    arrow->showMenu();
    
    //mainmenu->deleteLater();
    //delete mainmenu;
    //mainmenu = NULL;
}


void PathLine::goTo(QTreeWidgetItem * item, int column )
{
    QString path = buttonsPath2.value(current_button_pressed);
    model->setRootPath(path + item->text(0));
    QDir::setCurrent(path + item->text(0));
    mainmenu->deleteLater();
    
    for (int i=0; i < buttons.length(); i++)
        removeWidget(buttons.at(i));
        
    buttons.clear();
    setText("");
    addButtons(path + item->text(0));
}

void PathLine::goTo()
{
    QPushButton * pbtn = qobject_cast<QPushButton*>(sender());
    QString path = buttonsPath.value(pbtn);
    
    model->setRootPath(path);
    QDir::setCurrent(path);
    
    for (int i=0; i < buttons.length(); i++)
        removeWidget(buttons.at(i));
        
    buttons.clear();
    setText("");
    addButtons(path);
}

void PathLine::mousePressEvent(QMouseEvent  * event)
{
    for (int i=0; i < buttons.length(); i++)
        removeWidget(buttons.at(i));
    
    setText(path);
}

void PathLine::returnPressed()
{
    /*if ( highlighted ){
        highlighted = false;
        return;
    }*/
    
    if ( ! QFile::exists(text()) )
        return;
    
    path = text();
    model->setRootPath(path);
    QDir::setCurrent(path);
    focusOutEvent(0);
}

void PathLine::focusOutEvent(QFocusEvent * e)
{
        
    //set the old path as buttons
    buttons.clear();
    setText("");
    addButtons(path);
}

void PathLine::setVisible(bool p_visible)
{
    //this->setVisible(p_visible);
    QLineEdit::setVisible(p_visible);
    if (! p_visible)
    {
        //Clear the filter when it is hidden.
        clear();
    }
}

void PathLine::setModel(QFileSystemModel *current_model)
{
    model = current_model;
}

bool PathLine::event(QEvent *e)
{
    if (e->type() == QEvent::KeyPress){
        
        if (model->rootPath().compare(completer_model->rootPath()) != 0)
            completer_model->setRootPath(model->rootPath());
        
        
        QKeyEvent *ke = static_cast<QKeyEvent *>(e);
        if (ke->key() == Qt::Key_Return)
        {
            
            completer->popup()->hide();
            qDebug() << "PathLine event: enter pressed";
        }
    }
    return QLineEdit::event(e);
}

void PathLine::insertText(const QString & text)
{
    path = text;
    setText(path);
    //completer_model->setRootPath(text);
    for (int i=0; i < buttons.length(); i++)
        removeWidget(buttons.at(i));
    focusOutEvent(0);
}



