/* 
 *  Copyright (c) 2010 Martin Ribelotta
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "previewwidget.h"


#include <QImage>
#include <QImageReader>
#include <QThreadPool>
#include <QFileIconProvider>

#include <QtDebug>

#include "imagerenderer.h"


PreviewWidget::PreviewWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::PreviewWidget)
{
    iconProvider = new QFileIconProvider;
    pool = new QThreadPool(this);
    pool->setExpiryTimeout(5000);
    pool->setMaxThreadCount(2);
    ui->setupUi(this);
}

PreviewWidget::~PreviewWidget()
{
    delete iconProvider;
    delete ui;
}

void PreviewWidget::startPreview(const QFileInfo &info)
{
    if (info.isDir())
        ui->label->setPixmap(QPixmap("/usr/share/icons/oxygen/128x128/places/folder.png"));
    else if (QImageReader(info.absoluteFilePath()).canRead())
        pool->start(new ImageRenderer(ui->label->size(),
                                      info.absoluteFilePath(),
                                      this, SLOT(finished(QImage*))));
    else
        ui->label->setPixmap(iconProvider->icon(info).pixmap(128));
    showFileInfo(info);
}

static const double KBYTES = 1024.0;
static const double MBYTES = KBYTES * 1024.0;
static const double GBYTES = MBYTES * 1024.0;

static inline QString humanReadableSize(qulonglong cuantity)
{
    QString prefix;
    double factor = 1.0;
    if (cuantity<KBYTES) {
        prefix = QObject::tr("B");
        factor = 1.0;
    } else if (cuantity<MBYTES) {
        prefix = QObject::tr("KiB");
        factor = KBYTES;
    } else if (cuantity<GBYTES) {
        prefix = QObject::tr("MiB");
        factor = MBYTES;
    } else {
        prefix = QObject::tr("GiB");
        factor = GBYTES;
    }
    return QString("%1 %2").arg(cuantity/factor, 0, 'f', 2, QChar('0')).arg(prefix);
}

static inline QString humanReadableType(const QFileInfo& info)
{
    if (info.isDir())
        return QObject::tr("Directory");
    if (info.isBundle())
        return QObject::tr("Mac Bundled");
    if (info.isExecutable())
        return QObject::tr("Executable");
    return info.completeSuffix();
}

void PreviewWidget::showFileInfo(const QFileInfo &info)
{
    ui->textBrowser->setHtml(QString(
            "<html>"
            "<body>"
            "<div><h2>%1</h2></div>"
            "<div><b>File type: </b>%2</div>"
            "<div><b>Size: </b>%3</div>"
            "</body>"
            "</html>")
                             .arg(info.fileName())
                             .arg(humanReadableType(info))
                             .arg(humanReadableSize(info.size())));
}

void PreviewWidget::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void PreviewWidget::finished(QImage *tmpImage)
{
    ui->label->setPixmap(QPixmap::fromImage(*tmpImage));
    delete tmpImage;
}
